# solve the Poisson equation -Delta u = f
# with Dirichlet boundary condition u = 0

from ngsolve import *
from netgen.geom2d import unit_square

ngsglobals.msg_level = 1

uexact = 16*x*(1-x)*y*(1-y)

def SetUp(mesh):
    fes = H1(mesh, order=3, dirichlet="left|right|top|bottom")
    u,v = fes.TnT()
    
    # the right hand side
    f = LinearForm(fes)
    f += 32 * (y*(1-y)+x*(1-x)) * v * dx
    
    # the bilinear-form 
    a = BilinearForm(fes, symmetric=True)
    a += grad(u)*grad(v)*dx

    gfu = GridFunction(fes)

    return (gfu,a,f)


output = []
def Solve_Poisson(mesh, gfu, a, f):
    a.Assemble()
    f.Assemble()
    
    inv = a.mat.Inverse(freedofs=gfu.space.FreeDofs(), inverse="umfpack")
    res = f.vec.CreateVector()
    res.data = f.vec - a.mat*gfu.vec
    gfu.vec.data += inv * res

    #error
    u=gfu

    err_u =  sqrt (Integrate ( (u-uexact)**2, mesh))
    output.append ((gfu.space.ndof, err_u))
    print(output)

hh=0.2
for i in range(4):
    mesh = Mesh(unit_square.GenerateMesh(maxh=hh))
    hh=hh/2
    gfu, a, f = SetUp(mesh)
    Solve_Poisson(mesh, gfu, a, f)

i = 1
while i < len(output):
    order_u = log(output[i-1][1]/output[i][1])/log(2)
    
    print("%0.3f | %0.3e" %
      (order_u, output[i-1][1]))
    i =  i+1
